/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.managementcenter.project.web;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sg.managementcenter.project.entity.Project;
import com.jeeplus.modules.sg.managementcenter.project.service.ProjectService;
import com.jeeplus.modules.sg.managementcenter.project.util.ImportUtil;
import com.jeeplus.modules.sg.managementcenter.project.util.ProExcelInfo;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;

import static com.jeeplus.modules.sg.managementcenter.project.util.ProExcelInfo.getProjectUtilFlag;

/**
 * 表单Controller
 * @author lgf
 * @version 2018-06-12
 */
@Controller
@RequestMapping(value = "${adminPath}/sg/managementcenter/project")
public class ProjectController extends BaseController {

	@Autowired
	private ProjectService projectService;
	
	@ModelAttribute
	public Project get(@RequestParam(required=false) String id) {
		Project entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = projectService.get(id);
		}
		if (entity == null){
			entity = new Project();
		}
		return entity;
	}
	
	/**
	 * 工程管理表单列表页面
	 */
	@RequiresPermissions("sg:managementcenter:project:list")
	@RequestMapping(value = {"list", ""})
	public String list(Project project, Model model) {
		model.addAttribute("leave1", project);
		return "modules/sg/managementcenter/project/projectList";
	}
	
		/**
	 * 表单列表数据
	 */
	@ResponseBody
	@RequiresPermissions("sg:managementcenter:project:list")
	@RequestMapping(value = "data")
	public Map<String, Object> data(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
		return getBootstrapData(page);
	}

	@ResponseBody
	@RequiresPermissions("sg:managementcenter:project:list")
	@RequestMapping(value = "dataSys")
	public Map<String, Object> dataSys(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
		project.setStatus("0");
		project.setWhetherCloseStatus("0");
		Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
		return getBootstrapData(page);
	}
	/**
	 * 查看，增加，编辑表单表单页面
	 */
	@RequiresPermissions(value={"sg:managementcenter:project:view","sg:managementcenter:project:add","sg:managementcenter:project:edit"},logical= Logical.OR)
	@RequestMapping(value = "form")
	public String form(Project project, Model model) {
		model.addAttribute("leave1", project);
		return "modules/sg/managementcenter/project/projectForm";
	}

	/**
	 * 保存表单
	 */
	@ResponseBody
	@RequiresPermissions(value={"sg:managementcenter:project:add","sg:managementcenter:project:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public AjaxJson save(Project project, Model model) throws Exception{
		AjaxJson j = new AjaxJson();
		/**
		 * 后台hibernate-validation插件校验
		 */
		String errMsg = beanValidator(project);
		if (StringUtils.isNotBlank(errMsg)){
			j.setSuccess(false);
			j.setMsg(errMsg);
			return j;
		}
		//新增或编辑表单保存
		projectService.save(project);//保存
		j.setSuccess(true);
		j.setMsg("保存项目明细数据成功");
		return j;
	}
	
	/**
	 * 删除表单
	 */
	@ResponseBody
	@RequiresPermissions("sg:managementcenter:project:del")
	@RequestMapping(value = "delete")
	public AjaxJson delete(Project project) {
		AjaxJson j = new AjaxJson();
		projectService.delete(project);
		j.setMsg("删除项目明细数据成功");
		return j;
	}
	
	/**
	 * 批量表单
	 */
	@ResponseBody
	@RequiresPermissions("sg:managementcenter:project:del")
	@RequestMapping(value = "deleteAll")
	public AjaxJson deleteAll(String ids) {
		AjaxJson j = new AjaxJson();
		String idArray[] =ids.split(",");
		for(String id : idArray){
			projectService.delete(projectService.get(id));
		}
		j.setMsg("删除项目明细数据成功");
		return j;
	}
	
	/**
	 * 导出excel文件
	 */
	@ResponseBody
	@RequiresPermissions("sg:managementcenter:project:export")
    @RequestMapping(value = "export")
    public AjaxJson exportFile(Project project, HttpServletRequest request, HttpServletResponse response) {
		AjaxJson j = new AjaxJson();
		try {
            String fileName = "项目明细表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Project> page = projectService.findPage(new Page<Project>(request, response, -1), project);
    		new ExportExcel("项目明细表", Project.class).setDataList(page.getList()).write(response, fileName).dispose();
    		j.setSuccess(true);
    		j.setMsg("导出成功！");
    		return j;
		} catch (Exception e) {
			j.setSuccess(false);
			j.setMsg("导出项目明细表表单记录失败！失败信息："+e.getMessage());
		}
			return j;
    }

	/**
	 * 导入Excel数据

	 */
	@ResponseBody
	@RequiresPermissions("sg:managementcenter:project:import")
    @RequestMapping(value = "import")
   	public AjaxJson importFile(@RequestParam("file")MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
		AjaxJson j = new AjaxJson();
		try {
			ImportUtil importUtil = new ImportUtil(file, 0, 0);
			Boolean projectUtilFlag = ProExcelInfo.getProjectUtilFlag(importUtil);
			if (projectUtilFlag){
				j.setSuccess(false);
				j.setMsg("表不符合规范");
				return j;
			}
			List<Project> projects = ProExcelInfo.getProjectUtil(importUtil);
			projectService.saveList(projects);
			j.setMsg( "已成功导入 "+projects.size()+" 条项目表单记录");
		} catch (Exception e) {
			j.setSuccess(false);
			j.setMsg("导入项目表单失败！失败信息："+e.getMessage());
		}
		return j;
    }
}